/// <reference types="node" />
import * as net from 'net';
import { EventEmitter } from 'events';
export declare class TcpUnit extends EventEmitter {
    sockaddr: string;
    sock?: net.Socket;
    host: string;
    port: number;
    retryCount: number;
    private serverId;
    recv_pool: Buffer[];
    recv_len: number;
    constructor(sockadd: string, sock: net.Socket | undefined, host?: string, port?: number, serverId?: string);
    on(event: 'connection', listener: () => void): this;
    on(event: 'data', listener: (cmd: number, info: string | Buffer) => void): this;
    on(event: 'close', listener: (socketAddr: string, server_close: boolean) => void): this;
    on(event: 'error', listener: (socketAddr: string, error: Error) => void): this;
    reciveData(data: Buffer): void;
    resetData(data: Buffer): void;
    process(cmd: number, info: Buffer): void;
    doParse(): void;
    sendMsg(cmd: number, msg: string | Buffer): boolean;
    close(): void;
}
/**tcp 连接的管理，负责tcp连接的响应和处理 */
export declare class TcpServer extends EventEmitter {
    static serPools: {
        [sock: string]: TcpServer;
    };
    static create(port: number): TcpServer;
    static getServer(sockaddr: string): TcpServer;
    static isDispersedMode(sockaddr: string): boolean;
    sockaddr: string;
    private netPools;
    getUnit(sockaddr: string): TcpUnit;
    private isDispersedMode;
    constructor(port: number);
    on(event: 'connection', listener: (sockaddr: string) => void): this;
    on(event: 'data', listener: (sockaddr: string, cmd: number, info: string | Buffer) => void): this;
    on(event: 'close', listener: (sockaddr: string) => void): this;
    on(event: 'error', listener: (sockaddr: string, error: Error) => void): this;
    private onconnection;
    pushMsg(sockaddr: string, cmd: number, msg: string | Buffer): boolean;
    private onData;
    private onClose;
    private onError;
    close(): void;
}
export declare class TcpSocket {
    static create(host: string, port: number, count?: number): TcpUnit;
    private static _createSocket;
    static connect(sockUnit: TcpUnit): Promise<[string]>;
    static reConnect(sockUnit: TcpUnit): void;
}
