"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TcpNet_1 = require("../TcpNet");
const events_1 = require("events");
const debug_1 = require("debug");
const Time_1 = require("../Time");
const proto_1 = require("./proto");
let Debug = debug_1.debug('server-libs');
class GroupMgr {
    constructor(host, port) {
        this.host = '';
        this.port = 0;
        this.selfInfo = { roleId: '', score: 0, info: {} };
        this.groupId = '';
        this.groupInfo = { selfRole: proto_1.RoleType.normal, roles: {} };
        this.event = new events_1.EventEmitter();
        //-------------------------------------//
        this.hellback = {};
        this.host = host;
        this.port = port;
    }
    static create(host, port, roleId, groupId, score, info) {
        let group = new GroupMgr(host, port);
        group.joinGroup(roleId, groupId, score, info);
        return group;
    }
    init() {
        return new Promise((resolve, reject) => {
            this.socket = TcpNet_1.TcpSocket.create(this.host, this.port);
            TcpNet_1.TcpSocket.connect(this.socket).then(() => {
                this.onConnection();
                resolve();
            }).catch(reject);
        });
    }
    onConnection() {
        this.socket.removeAllListeners('connection');
        this.socket.on('close', this.onClose.bind(this));
        this.socket.on('data', this.onData.bind(this));
        this.socket.on("error", () => { });
        this.event.emit("connection");
    }
    get isMaster() {
        return this.groupInfo.selfRole == proto_1.RoleType.master;
    }
    // 查询某个节点的角色
    getRole(roleId) {
        for (let key in this.groupInfo.roles) {
            if (!this.groupInfo.roles[key])
                continue;
            if (this.groupInfo.roles[key].indexOf(roleId) >= 0) {
                return parseInt(key);
            }
        }
        return proto_1.RoleType.normal;
    }
    get masterRoleId() {
        return this.groupInfo.roles[proto_1.RoleType.master][0];
    }
    get roleId() {
        return this.selfInfo.roleId;
    }
    onsayhellotiemout() {
        try {
            // 通知所有节点失败的消息
            let pool = this.hellback.b || [];
            for (let i = 0; i < pool.length; i++) {
                pool[i][1]();
            }
        }
        catch (e) {
        }
        this.hellback.b = [];
    }
    // 通知client 应答状态
    sayhello(data) {
        if (!this._pushData(proto_1.eMasterC2SType.sayhello, JSON.stringify(data)))
            return Promise.reject();
        return new Promise((resolve, reject) => {
            if (!this.hellback.b) {
                this.hellback.b = [[resolve, reject]];
            }
            else {
                this.hellback.b.push([resolve, reject]);
            }
            if (!this.hellback.t) {
                this.hellback.t = setTimeout(this.onsayhellotiemout.bind(this), 5000);
            }
        });
    }
    //------------------------------------------//
    onData(cmd, data) {
        let info = JSON.parse(data.toString());
        switch (cmd) {
            // 注册信息
            case proto_1.eMasterS2CType.join:
                this.onJoin(info.roleId, info.role, info.groupId);
                break;
            case proto_1.eMasterS2CType.talk:
                this.onTalk(info.srcId, info);
                break;
            case proto_1.eMasterS2CType.groupInit:
                this.onInit(info.groupId, info.roles);
                break;
            case proto_1.eMasterS2CType.roleResult:
                this.onRoleResult(info.groupId, info.roles);
                break;
            case proto_1.eMasterS2CType.leave:
                this.onLeave(info.roleId, info.groupId, info.role);
                break;
            case proto_1.eMasterS2CType.groupInitFail:
                // 重复了，就要提示用户
                this.event.emit("error", info.addr);
                break;
            case proto_1.eMasterS2CType.sayhello:
                this._pushData(proto_1.eMasterC2SType.sayhelloback, JSON.stringify({
                    t: Time_1.LocalDate.now(),
                    role: this.getRole(this.roleId)
                }));
                break;
            case proto_1.eMasterS2CType.sayhelloback: {
                try {
                    // 移除超时
                    if (this.hellback.t) {
                        clearTimeout(this.hellback.t);
                        delete this.hellback.t;
                    }
                    // 通知所有节点成功的消息
                    let pool = this.hellback.b || [];
                    for (let i = 0; i < pool.length; i++) {
                        pool[i][0](data);
                    }
                }
                catch (e) {
                }
                this.hellback.b = [];
                break;
            }
            default:
                break;
        }
    }
    // 断开的时候尝试一下解除
    onClose(socketaddr, isException) {
        let _this = this;
        // 把自己的group信息干掉
        // 把自己的数据设置为普通，防止master状态下执行
        this.groupInfo.selfRole = proto_1.RoleType.normal;
        // 清理掉所有人的角色信息
        for (let key in this.groupInfo.roles) {
            let pool = this.groupInfo.roles[key];
            if (pool)
                pool.splice(0, pool.length);
        }
        if (isException) {
            // 断开的时候需要 重新发起连接
            function connect() {
                TcpNet_1.TcpSocket.connect(_this.socket).then(_this.onConnection.bind(_this)).catch(() => {
                    Debug("net close:", socketaddr, "ready to auto reconnect");
                    process.nextTick(connect);
                });
            }
            connect();
        }
        // 上报自己的工作状态为断开
        this.event.emit("close");
    }
    close() {
        this.event.removeAllListeners();
        this.socket.removeAllListeners();
        this.socket.close();
    }
    // 加入分组
    joinGroup(roleId, groupId, score, info) {
        this.selfInfo.roleId = roleId;
        this.selfInfo.info = info;
        this.groupId = groupId;
        this.selfInfo.score = score || this.selfInfo.score;
        return this._pushData(proto_1.eMasterC2SType.join, JSON.stringify({
            groupId: this.groupId,
            roleId: this.selfInfo.roleId,
            info: Object.assign({ score: this.selfInfo.score }, this.selfInfo.info)
        }));
    }
    _pushData(cmd, data) {
        return this.socket.sendMsg(cmd, data);
    }
    // 推送点对点的消息
    pushMsg(targetId, info) {
        return this._pushData(proto_1.eMasterC2SType.talk, JSON.stringify({
            groupId: this.groupId,
            roleId: this.selfInfo.roleId,
            targetId: targetId,
            info: info
        }));
    }
    // 群发消息
    pushBroadcast(info) {
        return this._pushData(proto_1.eMasterC2SType.brodcast, JSON.stringify({
            groupId: this.groupId,
            roleId: this.selfInfo.roleId,
            info: info
        }));
    }
    // 收到分组初始化信息 
    onInit(groupId, roles) {
        this.groupId = groupId;
        this.parseRole(roles);
        // 上报自己状态为连接
        this.event.emit('open');
        this.event.emit('ready');
    }
    parseRole(roles) {
        let preRole = this.groupInfo.selfRole;
        this.groupInfo.roles = roles;
        let find = false;
        for (let key in roles) {
            if (!roles[key])
                continue;
            if (roles[key].indexOf(this.selfInfo.roleId) >= 0) {
                this.groupInfo.selfRole = parseInt(key);
                find = true;
                break;
            }
        }
        if (!find) {
            this.groupInfo.selfRole = proto_1.RoleType.normal;
        }
        // 这里对于选举结果要处理一下，如果自己的角色定位改变了需要额外通知一下
        if (preRole != this.groupInfo.selfRole) {
            this.event.emit("roletype", this.groupInfo.selfRole == proto_1.RoleType.master, this.groupInfo.selfRole);
        }
    }
    // 有人加入
    onJoin(roleId, role, groupId) {
        if (this.groupId != groupId)
            return;
        if (!this.groupInfo.roles[role]) {
            this.groupInfo.roles[role] = [roleId];
        }
        else {
            this.groupInfo.roles[role].push(roleId);
        }
    }
    // 有人离开
    onLeave(roleId, groupId, role) {
        if (this.groupId != groupId)
            return;
        if (this.groupInfo.roles[role]) {
            let idx = this.groupInfo.roles[role].push(roleId);
            this.groupInfo.roles[role].splice(idx, 1);
        }
    }
    // 重新选举了
    onRoleResult(groupId, roles) {
        if (this.groupId != groupId)
            return;
        this.groupId = groupId;
        this.parseRole(roles);
    }
    onTalk(roleId, info) {
        this.event.emit('data', roleId, info);
    }
    on(event, listener) { this.event.on(event, listener); return this; }
    once(event, listener) { this.event.once(event, listener); return this; }
    off(event, listener) { this.event.off(event, listener); return this; }
}
exports.GroupMgr = GroupMgr;
