"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("./client");
const ResopnCache_1 = require("./ResopnCache");
const server_1 = require("./server");
var MasterService = {
    /**
       * 注册request响应，一般都是向master请求 解决竞争资源的设计
       * @param cmd
       * @param cb
       */
    regist: function (cmd, cb) {
        MasterSlaveMgr.push_cmd('request', cmd, cb);
    }
};
var SlaverService = {
    /**
      * 注册control响应,一般都是节点之间的消息通信
      * @param cmd
      * @param cb
      */
    regist: function (cmd, cb) {
        MasterSlaveMgr.push_cmd('control', cmd, cb);
    },
    /** 提供给节点间通信，主要例如 告诉其它节点需要释放掉某个用户的缓存
     * @param cmd ControlCmd
     * @param info 传递的数据
    */
    broadNotice: function (cmd, info, toSelf = false) {
        return MasterSlaveMgr.net.normalBroadcast(cmd, info, toSelf);
    },
    /**向master请求内容
     * @param cmd RequestCmd
     * @param info 传递的数据
    */
    masterRequest: function (cmd, info) {
        return new Promise(function (resolve, reject) {
            let requestId = ResopnCache_1.ResopnCache.createId();
            if (!MasterSlaveMgr.net.requestMsg(MasterSlaveMgr.net.masterId, cmd, info, requestId)) {
                reject({ code: 4 });
            }
            else {
                ResopnCache_1.ResopnCache.addCache(requestId, function (info) {
                    if (info.code == 0) {
                        resolve(info);
                    }
                    else {
                        reject(info);
                    }
                });
            }
        });
    }
};
class MasterSlaveMgr {
    static get net() {
        return this._net;
    }
    static push_cmd(type, cmd, cb) {
        if (!this.eventHandle[type]) {
            this.eventHandle[type] = {};
        }
        if (!this.eventHandle[type][cmd]) {
            this.eventHandle[type][cmd] = [cb];
        }
        else {
            this.eventHandle[type][cmd].push(cb);
        }
    }
    static call_cmd(t, type, cmd, info, role, requestId) {
        if (!this.eventHandle[type]) {
            return;
        }
        if (!this.eventHandle[type][cmd]) {
            return;
        }
        let list = this.eventHandle[type][cmd];
        for (let i = 0; i < list.length; i++) {
            if (type == 'request') {
                let cb = list[i];
                cb(info, role, requestId || "").then(v => {
                    t.answerMsg(role, 0, v, requestId);
                }).catch(e => {
                    if (e instanceof Error) {
                        e = { code: 5, errMsg: e.stack || e.message };
                    }
                    else if (typeof e == 'string') {
                        e = { code: 5, errMsg: e };
                    }
                    else if (e && e.code == undefined) {
                        e.code = 5;
                    }
                    else if (e == undefined) {
                        e = { code: 5 };
                    }
                    t.answerMsg(role, 0, e, requestId);
                });
            }
            else if (type == 'control') {
                try {
                    list[i].apply(t, [info, role, requestId]);
                }
                catch (e) {
                    console.error(e);
                }
            }
        }
    }
    static init(host, port, id, groupId, score, info) {
        if (this.net)
            this.net.close();
        this._net = new client_1.MasterSlaver(host, port);
        this.net.on('request', this.onRequest);
        this.net.on('answer', this.onAnswer);
        this.net.on('control', this.onControl);
        return this.net.init(id, groupId, score, info);
    }
    /**接收 request 请求 */
    static onRequest(cmd, info, role, requestId) {
        MasterSlaveMgr.call_cmd(this, 'request', cmd, info, role, requestId);
    }
    /**接收 control 请求  */
    static onControl(cmd, info, role, requestId) {
        MasterSlaveMgr.call_cmd(this, 'control', cmd, info, role, requestId);
    }
    /**接收 answer 请求 */
    static onAnswer(cmd, info, role, requestId) {
        if (requestId && ResopnCache_1.ResopnCache.onRecive(requestId, info)) {
            return;
        }
    }
}
MasterSlaveMgr.eventHandle = {};
exports.MasterSlaverMoudle = {
    /**client的master角色 */
    master: MasterService,
    /**client的slaver角色 */
    slaver: SlaverService,
    /**client的初始化角色 */
    init: MasterSlaveMgr.init.bind(MasterSlaveMgr),
    beMaster: function (func) {
        if (MasterSlaveMgr.net) {
            MasterSlaveMgr.net.on('roletype', func);
            return true;
        }
        return false;
    },
    instance() {
        if (MasterSlaveMgr.net == undefined)
            throw Error("need  MasterSlaverMoudle.init first");
        return MasterSlaveMgr.net;
    },
    isMaster() {
        if (MasterSlaveMgr.net == undefined)
            throw Error("need MasterSlaverMoudle.init first");
        return MasterSlaveMgr.net.isMaster;
    },
    roleId() {
        if (MasterSlaveMgr.net == undefined)
            throw Error("need  MasterSlaverMoudle.init first");
        return MasterSlaveMgr.net.roleId;
    },
    /**server的实例 */
    serverInstance: server_1.MasterNet,
    name: 'MasterSlaverMoudle'
};
