"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TcpNet_1 = require("../TcpNet");
const proto_1 = require("./proto");
const Time_1 = require("../Time");
const events_1 = require("events");
class Master extends events_1.EventEmitter {
    constructor() {
        super();
        //-------------------------------------//
        this.hellback = {};
    }
    init(port) {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = TcpNet_1.TcpServer.create(port);
            this.server.on('connection', this.onConnection.bind(this));
            this.server.on('data', this.onData.bind(this));
            this.server.on('close', this.onClose.bind(this));
            return true;
        });
    }
    onConnection(sockaddr) {
        // 这里以后要增加一个超时检查
        this.emit('open', sockaddr);
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    once(event, listener) {
        return super.once(event, listener);
    }
    pushData(sockaddr, cmd, data) {
        return this.server ? this.server.pushMsg(sockaddr, cmd, data) : false;
    }
    // 一个5s的应答超时机制
    sayhelloTimeout(sockaddr) {
        return setTimeout(() => {
            if (!this.hellback[sockaddr])
                return;
            try {
                // 通知所有节点失败的消息
                let pool = this.hellback[sockaddr].b;
                for (let i = 0; i < pool.length; i++) {
                    pool[i][1]();
                }
            }
            catch (e) {
            }
            delete this.hellback[sockaddr];
        }, 5000);
    }
    // 通知client 应答状态
    sayhello(sockaddr, data) {
        if (!this.pushData(sockaddr, proto_1.eMasterS2CType.sayhello, JSON.stringify(data)))
            return Promise.reject();
        return new Promise((resolve, reject) => {
            if (!this.hellback[sockaddr]) {
                this.hellback[sockaddr] = {
                    b: [[resolve, reject]],
                    t: this.sayhelloTimeout(sockaddr)
                };
            }
            else {
                this.hellback[sockaddr].b.push([resolve, reject]);
            }
        });
    }
    //------------------------------------------//
    onData(sockaddr, cmd, data) {
        let info = JSON.parse(data.toString());
        switch (cmd) {
            // 注册信息
            case proto_1.eMasterC2SType.sayhello:
                this.pushData(sockaddr, proto_1.eMasterS2CType.sayhelloback, JSON.stringify({ t: Time_1.LocalDate.now() }));
                break;
            case proto_1.eMasterC2SType.sayhelloback: {
                if (!this.hellback[sockaddr])
                    break;
                try {
                    // 移除超时
                    clearTimeout(this.hellback[sockaddr].t);
                    // 通知所有节点成功的消息
                    let pool = this.hellback[sockaddr].b;
                    for (let i = 0; i < pool.length; i++) {
                        pool[i][0](data);
                    }
                }
                catch (e) {
                }
                delete this.hellback[sockaddr];
                break;
            }
            default:
                this.emit('data', cmd, sockaddr, info);
                break;
        }
    }
    // 断开的时候尝试一下解除
    onClose(sockaddr) {
        this.emit('close', sockaddr);
    }
}
exports.Master = Master;
exports.MasterNet = new Master();
