"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const MONGO = __importStar(require("mongodb"));
const debug_1 = require("debug");
const Util_1 = require("../Util");
let Debug = debug_1.debug('server-libs');
function error(...args) {
    Debug('%O', ...args);
}
function func_attach(dst, src) {
    if (!dst || !src)
        return;
    for (var key in src) {
        dst[key] = Util_1.Util.copy(src[key]);
    }
    return dst;
}
class ReHash extends events_1.EventEmitter {
    constructor(parent, fObj, judgefunc, args) {
        super();
        this.fObj = {};
        this._data_ = {};
        this._online_ = false;
        this.ready = false;
        this._changeKey = [];
        this._auto_save = 0;
        this._reload_count = 0;
        this._cc_ = null;
        this._judge_list = [];
        this._bind_auto_save = this.auto_save.bind(this);
        this.fObj = Util_1.Util.copy(fObj);
        this._data_ = Util_1.Util.copy(fObj);
        this.args = args;
        this._collection_ = parent;
        this._jugetfunc = judgefunc;
        this.on('db_loaded', this._on_load.bind(this));
    }
    get findKey() {
        return this.fObj;
    }
    setObjectID(objID, v) {
        this._data_ = Object.assign(this._data_ || {}, { _id: objID }, this.findKey, v || {});
    }
    force_ok() {
        this._online_ = this._data_._id ? true : false;
    }
    insert(v) {
        return new Promise((resolve, reject) => {
            // 这里把强制的 findkey 也复制上去
            v = func_attach(v, this.findKey);
            this._collection_.insertOne(v, (err, res) => {
                if (err)
                    reject(err);
                else {
                    if (this._online_) {
                        if (res.result.ok && res.ops.length) {
                            this._data_ = res.ops[0];
                        }
                    }
                    resolve();
                }
            });
        });
    }
    /**
     * 返回的都是 拷贝项，变更都要主动set
     * @param key
     */
    get(key) {
        if (!this._data_)
            return null;
        return Util_1.Util.copy(this._get(key));
    }
    _get(key) {
        let sKey = key.split('.');
        let obj = this._data_;
        for (let i = 0; i < sKey.length; i++) {
            let oneKey = sKey[i];
            if (i == sKey.length - 1) {
                return obj[oneKey];
            }
            else if (!obj.hasOwnProperty(oneKey)) {
                break;
            }
            else {
                obj = obj[oneKey];
            }
        }
        return undefined;
    }
    _set(key, value) {
        let sKey = key.split('.');
        let obj = this._data_;
        for (let i = 0; i < sKey.length; i++) {
            let oneKey = sKey[i];
            if (i == sKey.length - 1) {
                if (obj[oneKey] == value)
                    return false;
                if (value == undefined || value == null) {
                    delete obj[oneKey];
                }
                else {
                    obj[oneKey] = value;
                }
            }
            else {
                if (!obj.hasOwnProperty(oneKey)) {
                    obj[oneKey] = {};
                }
                obj = obj[oneKey];
            }
        }
        return true;
    }
    set(key, value) {
        if (!this._online_)
            return;
        if (this.findKey.hasOwnProperty(key))
            return;
        // 如果当前数据是空的，表示之前是没有这个数据的，需要把查询头加上
        if (!this._data_) {
            this._data_ = func_attach({}, this.findKey);
            this._collection_.save(this._data_);
        }
        if (this._get(key) == value)
            return;
        this._set(key, value);
        this.flag(key);
    }
    mset(a) {
        for (var i = 0; i < a.length; i++)
            this.set(a[i].k, a[i].v);
    }
    /**
     * 按照一个key 一个 value的方式传入参数
     * @param args
     */
    lset(...args) {
        let count = Math.ceil(args.length / 2);
        for (let i = 0; i < count; i++) {
            this.set(args[i * 2], args[i * 2 + 1]);
        }
    }
    /**
     * 标记一个字段需要重新保存一下
     * @param key
     */
    flag(key) {
        if (this._changeKey.indexOf(key) < 0)
            this._changeKey.push(key);
        this._check_save();
    }
    _check_save() {
        if (!this._data_ || this._auto_save)
            return;
        this._auto_save = setTimeout(this._bind_auto_save, 1000);
    }
    /**
     * 立即保存数据到数据库
     */
    force_save() {
        if (this._auto_save)
            clearTimeout(this._auto_save);
        return this.auto_save();
    }
    auto_save() {
        this._auto_save = 0;
        if (!this._data_)
            return Promise.resolve();
        var sets = {};
        var unsets = {};
        for (var i = 0; i < this._changeKey.length; i++) {
            let ckey = this._changeKey[i];
            let cvalue = this._get(ckey);
            if (cvalue == null || cvalue == undefined) {
                unsets[ckey] = 1;
            }
            else {
                sets[ckey] = cvalue;
            }
        }
        this._changeKey = [];
        var up_info = {};
        if (Object.keys(sets).length > 0) {
            up_info['$set'] = sets;
        }
        if (Object.keys(unsets).length > 0) {
            up_info['$unset'] = unsets;
        }
        // 拷贝一下 _id 防止更新错数据
        if (!this.findKey['_id'] && this._data_['_id']) {
            this.findKey['_id'] = this._data_['_id'];
        }
        if (Object.keys(up_info).length > 0) {
            return this._collection_.updateOne(this.findKey, up_info);
        }
        return Promise.resolve();
    }
    /**
     * 从数据库加载数据
     * @param fitle
     */
    load(fitle) {
        let _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this.ready = false;
                _this._judge_list = [];
                _this._online_ = true;
                // 外网出现数据重复的情况，这里需要优化一下
                _this._cc_ = _this._collection_.find(_this.findKey, { projection: fitle });
                _this.batchFind(resolve);
            }
            catch (e) {
                reject(e);
            }
        });
    }
    /**
     * 是否是空数据
     */
    get empty() {
        if (this._data_)
            return false;
        return true;
    }
    // 获取原始数据 如果改动的话会造成 set函数失败
    get data() {
        return this._data_;
    }
    // 获取一个复制后的数据备份
    get value() {
        var ot = Util_1.Util.copy(this._data_);
        delete ot['_id'];
        return ot;
    }
    // 获取一个保函 _id 的数据备份
    get value_all() {
        return Util_1.Util.copy(this._data_);
    }
    /**
     * 删除某个字段
     * @param key
     */
    del(key) {
        if (key instanceof Array) {
            for (var i = 0; i < key.length; i++)
                this.set(key[i], undefined);
        }
        else {
            this.set(key, undefined);
        }
    }
    /**
     * 删除当前数据
     */
    remove() {
        if (!this._online_)
            return;
        this._collection_.remove({ _id: this._data_._id });
    }
    _on_load(cb, err, data) {
        this.ready = true;
        if (err) {
            if (this._reload_count > 0) {
                error('db load error ', JSON.stringify(err, null, 4));
            }
            else {
                this.load().then(cb);
            }
            this._reload_count++;
        }
        else if (data.length == 1) {
            this._data_ = data[0] || null;
            if (cb)
                cb(this);
        }
        else if (data.length == 0) {
            this._data_ = null;
            if (cb)
                cb(this);
        }
        else {
            // 这里需要竞争一下，出现重复数据了
            if (this._jugetfunc) {
                //如果有排序的话使用自定义的，否则是用id较小的 
                data.sort(this._jugetfunc);
            }
            else {
                data.sort(function (a, b) {
                    return a['_id'] < b['_id'] ? -1 : 1;
                });
            }
            this._data_ = data[0];
            if (cb)
                cb(this);
        }
    }
    batchFind(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._cc_) {
                this.emit('db_loaded', cb, null, this._judge_list);
                return;
            }
            try {
                let count = 0;
                while (true) {
                    let result = yield this._cc_.next();
                    if (result == null) {
                        break;
                    }
                    else {
                        count++;
                        if (count == 2) {
                            // 第五次的时候打印一下,方便后续查错
                            console.error('batchFind error', JSON.stringify(this.findKey, null, 4), result);
                        }
                        this._judge_list.push(result);
                    }
                }
                this.emit('db_loaded', cb, null, this._judge_list);
            }
            catch (e) {
                error('db load error batchFind', JSON.stringify(e, null, 4));
                this.emit('db_loaded', cb, e, this._judge_list);
            }
        });
    }
}
exports.ReHash = ReHash;
class ReList {
    constructor(parent, fobj, args) {
        this.fobj = {};
        this._data_ = [];
        this._online_ = false;
        this._cc_ = null;
        this._start = 0;
        this._limit = 0;
        this.fobj = fobj;
        this._collection_ = parent;
        this.args = args;
    }
    get findKey() {
        return this.fobj;
    }
    get value() {
        var ot = Util_1.Util.copy(this._data_);
        for (var i = 0; i < ot.length; i++) {
            delete ot[i]['_id'];
        }
        return ot;
    }
    get value_all() {
        var ot = Util_1.Util.copy(this._data_);
        return ot;
    }
    set(index, key, value) {
        if (!this._online_)
            return;
        var s_v = this.get_at(index, true);
        if (!s_v)
            return;
        // 找到里面的
        if (s_v[key] == value)
            return;
        if (value == undefined) {
            delete s_v[key];
        }
        else {
            s_v[key] = value;
        }
        this._collection_.save(s_v);
    }
    push_back(v) {
        v = func_attach(v, this.findKey);
        this._data_.push(v);
        this._collection_.insertOne(v, (err, res) => {
            let idx = this._data_.length;
            if (!this._online_)
                return;
            if (res.result.ok && res.ops.length) {
                this._data_[idx - 1] = res.ops[0];
            }
            else {
                this._data_.splice(idx - 1, 1);
            }
        });
    }
    limit(num) {
        this._limit = num;
        return this;
    }
    skip(num) {
        this._start = num;
        return this;
    }
    load(desc = false, filter) {
        let _this = this;
        return new Promise(function (resolve, reject) {
            _this._online_ = true;
            try {
                _this._cc_ = _this._collection_.find(_this.findKey, { projection: filter });
                if (_this._start)
                    _this._cc_ = _this._cc_.skip(_this._start);
                _this.batchFind(resolve);
            }
            catch (e) {
                reject(e);
            }
        });
    }
    get length() {
        return this._data_.length;
    }
    get_at(index, org = false) {
        if (index < 0 || index >= this._data_.length)
            return null;
        if (org)
            return this._data_[index];
        return Util_1.Util.copy(this._data_[index]);
    }
    rm_at(index) {
        if (index < 0 || index >= this._data_.length)
            return;
        var rp = this._data_.splice(index, 1);
        for (var i = 0; i < rp.length; i++) {
            this._collection_.remove({ _id: rp[i]['_id'] });
        }
    }
    clear_all() {
        while (this.length > 0) {
            this.rm_at(0);
        }
    }
    _has_same(v, fobj) {
        var bSame = true;
        for (var key in fobj) {
            if (v[key] != fobj[key]) {
                bSame = false;
                break;
            }
        }
        return bSame;
    }
    del(fObj) {
        var idx = -1;
        for (var i = 0; i < this._data_.length; i++) {
            var c = this._data_[i];
            if (this._has_same(c, fObj)) {
                idx = i;
                break;
            }
        }
        // var idx = this._data_.indexOf(v);
        if (idx >= 0)
            this.rm_at(idx);
    }
    batchFind(cb) {
        this._cc_ && this._cc_.next((err, result) => {
            if (err) {
                if (cb)
                    cb(this, err);
                error('db load error batchFind', JSON.stringify(err, null, 4));
            }
            else {
                if (result == null) {
                    this._on_load(cb);
                }
                else {
                    this._data_.push(result);
                    if (this._limit && this._data_.length >= this._limit) {
                        this._on_load(cb);
                    }
                    else {
                        this.batchFind(cb);
                    }
                }
            }
        });
    }
    _on_load(cb) {
        if (cb)
            cb(this, null);
    }
}
exports.ReList = ReList;
class TeMongoDBUnit extends events_1.EventEmitter {
    constructor(db, collections = [], col_indexs) {
        super();
        this.index_check_pool = [];
        this.collections = [];
        this._collect_pool = {};
        this.ready = false;
        this._judge_count = 0;
        this._ct_wait_list = [];
        this._idx_wait_list = [];
        this.db = db;
        this.index_check_pool = col_indexs || [];
        this.collections = collections;
        this.db.collections(this._check_collections.bind(this));
        this.db.on('error', function (...s) {
            error('TeMongoDBUnit', JSON.stringify(s, null, 4));
        });
    }
    check_ready() {
        if (this.ready) {
            setTimeout(this.emit.bind(this, 'ready'), 10);
        }
    }
    _judge_index_finish() {
        if (this._judge_count > 0) {
            return;
        }
        // 都检查完了那么要创建不存在的index了
        // 检查一下要求的是否都在了
        for (var i = 0; i < this.collections.length; i++) {
            var rc = this.collections[i];
            if (rc && !this._collect_pool.hasOwnProperty(rc)) {
                this._ct_wait_list.push(rc);
                this.db.createCollection(rc, this._col_create_ret.bind(this, rc));
            }
        }
        this._create_index();
    }
    _col_create_ret(rc, error, succ) {
        let idx = this._ct_wait_list.indexOf(rc);
        if (idx >= 0) {
            this._ct_wait_list.splice(idx, 1);
        }
        this._create_index();
    }
    _create_index() {
        if (this._ct_wait_list.length != 0)
            return;
        // 表单创建好了，建立索引
        for (let i = 0; i < this.index_check_pool.length; i++) {
            let r = this.index_check_pool[i];
            if (this.collections.indexOf(r.col) >= 0) {
                this._check_index_keys(r);
                let wait_key = r.col + '_' + r.index;
                this._idx_wait_list.push(wait_key);
                this.db.createIndex(r.col, r.keys, this._col_idx_create_ret.bind(this, wait_key));
            }
        }
        this._init_finish();
    }
    _col_idx_create_ret(ct_str, err, result) {
        let idx = this._idx_wait_list.indexOf(ct_str);
        if (idx >= 0) {
            this._idx_wait_list.splice(idx, 1);
        }
        this._init_finish();
    }
    _check_index_keys(r) {
        if (!r.keys) {
            r.keys = {};
            let arr_keys = r.index.split('_');
            for (let j = 0; j < arr_keys.length; j = j + 2) {
                if (j + 1 >= arr_keys.length) {
                    r.keys[arr_keys[j]] = 1;
                }
                else {
                    let nb = parseInt(arr_keys[j + 1]);
                    if (nb == arr_keys[j + 1]) {
                        r.keys[arr_keys[j]] = nb;
                    }
                    else {
                        r.keys[arr_keys[j]] = arr_keys[j + 1];
                    }
                }
            }
        }
    }
    _judge_index(col, name, objs) {
        let keys = Object.keys(objs);
        for (let i = 0; i < this.index_check_pool.length; i++) {
            let r = this.index_check_pool[i];
            if (r.col != col)
                continue;
            this._check_index_keys(r);
            let same = true;
            for (let j = 0; j < keys.length; j++) {
                if (!r.keys.hasOwnProperty(keys[j])) {
                    same = false;
                    break;
                }
            }
            if (same) {
                this.index_check_pool.splice(i, 1);
                i--;
            }
        }
    }
    /**
     * 检查数据集是否存在
     */
    _check_collections(a, b) {
        if (a) {
            this.emit('error', a);
        }
        else {
            for (var i = 0; i < b.length; i++) {
                var bc = b[i];
                this._collect_pool[bc.collectionName] = true;
                this._judge_count++;
                bc.indexes(((col, error, indexs) => {
                    for (let i = 0; i < indexs.length; i++) {
                        let name = indexs[i].name;
                        /**默认 索引不需要判断 */
                        if (name == '_id_')
                            continue;
                        this._judge_index(col, indexs[i].name, indexs[i].key);
                    }
                    this._judge_count--;
                    this._judge_index_finish();
                }).bind(this, bc.collectionName));
            }
            this._judge_index_finish();
        }
    }
    _init_finish() {
        if (this._ct_wait_list.length != 0 || this._idx_wait_list.length != 0)
            return;
        // 表单和索引都建立好了，那么就完成初始化了
        this.ready = true;
        this.emit('ready');
    }
    get_unit(col_name, fObj = {}, jugetfunc, ...args) {
        return new ReHash(this.db.collection(col_name), fObj, jugetfunc, args);
    }
    get_list(col_name, fObj = {}, ...args) {
        return new ReList(this.db.collection(col_name), fObj, args);
    }
    /**
     * 直接插入数据，如果_id重复就异常
     * @param col_name
     * @param fObj
     * @param insert_value
     */
    insert(col_name, insert_value) {
        return this.get_collection(col_name).insertOne(insert_value);
    }
    /**
     * 只有不存在的时候才会插入新数据
     * @param col_name
     * @param fObj
     * @param insert_value
     */
    find_insert_unit(col_name, fObj = {}, insert_value) {
        return __awaiter(this, void 0, void 0, function* () {
            let db = this.get_unit(col_name, fObj, undefined);
            let s = yield db.load();
            if (s.empty) {
                yield s.insert(this.merageInfo(insert_value));
            }
            return true;
        });
    }
    update_insert(col_name, fObj = {}, insert_value) {
        return this.update(col_name, fObj, insert_value, true);
    }
    /**合并内容 */
    merageInfo(value) {
        let insert_value = {};
        for (let key in value) {
            let _keys = key.split('.');
            let _obj = insert_value;
            for (let i = 0; i < _keys.length; i++) {
                let __key = _keys[i];
                if (i == _keys.length - 1) {
                    _obj[__key] = value[key];
                }
                else {
                    if (!_obj[__key])
                        _obj[__key] = {};
                    _obj = _obj[__key];
                }
            }
        }
        return insert_value;
    }
    /**
     * 更新数据
     * @param col_name
     * @param fObj
     * @param value
     * @param insert 如果是 false的时候，数据不存在也返回操作成功
     */
    update(col_name, fObj = {}, value, insert = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let db = this.get_unit(col_name, fObj, undefined);
            let s = yield db.load();
            if (s.empty) {
                if (insert) {
                    // 这里应为什么都没有,需要合并对象
                    yield s.insert(this.merageInfo(value));
                }
                return true;
            }
            for (let key in value) {
                if (value[key] != s.get(key)) {
                    s.set(key, value[key]);
                }
            }
            yield s.force_save();
            return true;
        });
    }
    get_collection(table) {
        return this.db.collection(table);
    }
    get_collection_size(dbname, cb) {
        let col = this.db.collection(dbname);
        col.stats((function (_cb, err, res) {
            if (err) {
                _cb('err');
                error(err.toString);
            }
            else {
                _cb(JSON.stringify(res));
            }
        }).bind(this, cb));
    }
    get_aggregate(dbname, pipeline) {
        return new Promise((resolve, reject) => {
            this.db.collection(dbname).aggregate(pipeline, function (err, cur) {
                var outinfos = [];
                function batchFind() {
                    cur.next(function (_err, _res) {
                        if (_err) {
                            reject(_err);
                        }
                        else {
                            if (_res == null) {
                                resolve(outinfos);
                            }
                            else {
                                outinfos.push(_res);
                                batchFind();
                            }
                        }
                    });
                }
                batchFind();
            });
        });
    }
    /**
     * 检查和跟新存储函数
     * @param func
     */
    check_func(func) {
        let _this = this;
        return new Promise(function (resolve, reject) {
            // 先观察方法是否存在
            let fun_name = func.name;
            let dbscript = `db.system.js.save({'_id':'${fun_name}','value':${func.toString()}})`;
            function onSave() {
                _this.eval(dbscript).then(function (result) {
                    // result.retval = {nInserted:0 nMatched:0 nModified:0 nRemoved:0 nUpserted:1}
                    // 一般都是成功的，这里暂时不处理了
                    resolve();
                }).catch(function (err) {
                    reject(err.errmsg);
                });
            }
            _this.eval(fun_name).then(function (result) {
                if (result.retval && result.retval._bsontype == 'Code' && result.retval.code == func.toString()) {
                    // 达成要求了，不需要再操作了
                    resolve();
                }
                else {
                    // 发送命令
                    onSave();
                }
            }).catch(function (e) {
                onSave();
            });
        });
    }
    call_func(fun_name, ...args) {
        let options = {};
        // if (options['nolock']) {
        //     cmd['nolock'] = options['nolock'];
        // }
        for (let i = 0; i < args.length; i++) {
            if (typeof args[i] == 'string') {
                args[i] = '"' + args[i] + '"';
            }
        }
        return this.eval(`${fun_name}(${args.join(',')})`);
    }
    eval(cmd) {
        return this.db.command({ '$eval': `${cmd}` });
    }
    del(table, fobj) {
        return this.db.collection(table).deleteMany(fobj);
    }
}
exports.TeMongoDBUnit = TeMongoDBUnit;
class TeMongoDB extends events_1.EventEmitter {
    constructor() {
        super();
        this._gl_opts = {};
        this._pre_inits = [];
        // protected db: MONGO.Db;
        this._state_ = 'close';
        this._update_time = 0;
        this._last_error = '';
        this.db_maps = {};
        this.db_keys = [];
        this.mongodb = new MONGO.MongoClient('');
    }
    get ready() {
        return this._state_ == 'ready';
    }
    get state() {
        return this._state_;
    }
    set state(v) {
        this._state_ = v;
        this._update_time = Date.now();
    }
    get sys_state() {
        return {
            state: this._state_,
            up_time: this._update_time,
            last_err: this._last_error
        };
    }
    /**
     *
     * @param database_name 数据库的名字
     * @param collections 需要的数据集 可以选，选择后会生成对应的数据集后返回准备状态
     */
    get_database(database_name, collections = [], col_indexs) {
        if (!this.db_maps.hasOwnProperty(database_name)) {
            var opts = {};
            if (this.mongodb instanceof MONGO.ReplSet) {
                this.db_maps[database_name] = new TeMongoDBUnit(new MONGO.Db(database_name, this.mongodb, this._gl_opts), collections, col_indexs || []);
            }
            else {
                this.db_maps[database_name] = new TeMongoDBUnit(this.mongodb.db(database_name, opts), collections, col_indexs || []);
            }
            this.db_keys = Object.keys(this.db_maps);
        }
        var map = this.db_maps[database_name];
        map.check_ready();
        return map;
    }
    get_collection_size(db, col, cb) {
        let db_ = this.get_database(db);
        db_.get_collection_size(col, cb);
    }
    get defaultBDName() {
        if (this.db_keys.length > 0) {
            return this.db_keys[0];
        }
        else {
            return '';
        }
    }
    error_listen() {
        let events = [
            {
                name: 'timeout', func: () => {
                    this.state = 'timeout';
                    error('timeout');
                    this.emit('state', this.state);
                }
            },
            {
                name: 'error', func: (e) => {
                    if (e) {
                        this._last_error = e.toString();
                        error(this._last_error);
                        this.state = 'error';
                        this.emit('state', this.state);
                    }
                }
            },
            {
                name: 'close', func: () => {
                    this.state = 'close';
                    this.emit('state', this.state);
                }
            },
            {
                name: 'joined', func: (type, db) => {
                    if (type == 'primate' && this.state == 'left_primate') {
                        this.state = 'ready';
                        this.emit('state', this.state);
                    }
                },
            },
            {
                name: 'left', func: (type, db) => {
                    if (type == 'primate') {
                        this.state = 'left_primate';
                        error('left_primate');
                        this.emit('state', this.state);
                    }
                }
            }
        ];
        events.forEach(element => {
            this.mongodb.on(element.name, element.func);
        });
    }
    init(hosts, options = {}, pre_inits) {
        options.autoReconnect = true;
        var uri = '';
        if (hosts instanceof Array) {
            uri += 'mongodb://';
            if (options.auth) {
                uri += `${options.auth.user}:${options.auth.password}@`;
            }
            let server_lists = [];
            if (hosts.length == 1) {
                let { host: host, port: port } = hosts[0];
                server_lists.push(`${host}:${port}`);
            }
            else {
                for (var i = 0; i < hosts.length; i++) {
                    let { host: host, port: port } = hosts[i];
                    server_lists.push(`${host}:${port}`);
                }
            }
            uri += server_lists.join(',');
            if (options.auth) {
                uri += `/${options['user_db'] || 'admin'}`;
            }
            if (options.replicaSet) {
                uri += `?replicaSet=${options.replicaSet}`;
            }
        }
        else {
            uri = hosts;
        }
        // 'mongodb://root:chenkai@dds-bp1af52f28ac18141987-pub.mongodb.rds.aliyuncs.com:3717,dds-bp1af52f28ac18142178-pub.mongodb.rds.aliyuncs.com:3717/admin?replicaSet=mgset-7383357'
        this.mongodb = new MONGO.MongoClient(uri, Object.assign(options, { useUnifiedTopology: true, useNewUrlParser: true }));
        this._pre_inits = this._pre_inits.concat(pre_inits);
        // MONGO.Logger.setLevel("debug");
        this.mongodb['connect']((err, db) => {
            if (err) {
                this.state = 'error';
                this._last_error = err.toString();
                error(this._last_error);
                this.emit('error', err);
            }
            else {
                db.emit('open22');
            }
        });
        this.mongodb.on('open22', this._on_ready.bind(this));
        this.error_listen();
        return this;
    }
    _on_ready() {
        // return;
        for (var i = 0; i < this._pre_inits.length; i++) {
            var cf = this._pre_inits[i];
            var db = this.get_database(cf.database, cf.colltions, cf.indexs);
            db.on('ready', ((idx) => {
                this._pre_inits[idx].ready = true;
                this._check_ready();
            }).bind(this, i));
            db.on('error', (e) => {
                this._last_error = e.toString();
                this.state = 'error';
                error(this._last_error);
                this.emit('error', e);
            });
        }
        this._check_ready();
    }
    _check_ready() {
        var ok = true;
        for (var i = 0; i < this._pre_inits.length; i++) {
            var r = this._pre_inits[i];
            if (!r.ready) {
                ok = false;
                break;
            }
        }
        if (ok) {
            this.emit('ready');
            this.state = 'ready';
        }
    }
}
exports.TeMongoDB = TeMongoDB;
