"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const TeMongodb_1 = require("../mongodb/TeMongodb");
const crypto_1 = require("crypto");
const path_1 = require("path");
const CacheMoudle_1 = require("../CacheMoudle");
class ResourceDatabase {
    constructor(database) {
        this._databse = '';
        this._col = 'resource';
        this._db = new TeMongodb_1.TeMongoDB();
        this._databse = database;
    }
    init(path) {
        // 这里初始话一些数据库
        this._db.init(path, undefined, []);
        return new Promise((resolve) => {
            this._db.get_database(this._databse, [this._col]).once('ready', function () {
                resolve();
            });
        });
    }
    getResource(pack, file) {
        return __awaiter(this, void 0, void 0, function* () {
            let id = pack + '_' + file;
            let v = yield this._db.get_database(this._databse).get_unit(this._col, { _id: id }).load();
            if (v.empty)
                return Promise.reject();
            return v.data;
        });
    }
    saveResource(pack, file, data) {
        let id = pack + '_' + file;
        let md5 = crypto_1.createHash('md5').update(data).digest('hex');
        return this._db.get_database(this._databse).update(this._col, { _id: id }, { pack: pack, file: file, md5: md5, data: data }, true);
    }
}
class ResourceLocalFile {
    constructor() {
        this._path = '';
    }
    init(path) {
        this._path = path;
        if (!fs_1.default.existsSync(path)) {
            fs_1.default.mkdirSync(path, { recursive: true });
        }
        return;
    }
    getResource(pack, file) {
        return new Promise((resolve, reject) => {
            fs_1.default.readFile(path_1.join(this._path, pack, file), function (err, data) {
                if (err) {
                    reject();
                }
                else {
                    resolve({
                        name: file,
                        data: fs_1.default.readFileSync(file),
                        md5: crypto_1.createHash('md5').update(data).digest('hex')
                    });
                }
            });
        });
    }
    saveResource(pack, file, data) {
        return new Promise((resolve, reject) => {
            if (!fs_1.default.existsSync(path_1.join(this._path, pack))) {
                fs_1.default.mkdirSync(path_1.join(this._path, pack), { recursive: true });
            }
            fs_1.default.writeFile(path_1.join(this._path, pack, file), data, function (err) {
                if (err)
                    reject();
                else
                    resolve(true);
            });
        });
    }
}
class ResourceService {
    constructor() {
        // 暂定10分钟的缓存
        this._cache = CacheMoudle_1.CacheMoudle.createCache("ResourceService", 10 * 60 * 1000);
    }
    /**
     * 注册资源地址，可以是数据库或者本地目录
     * @param type
     * @param path
     */
    init(type, path, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (type == 'databse') {
                this._local_ = new ResourceDatabase(args[0]);
            }
            else if (type == 'file') {
                this._local_ = new ResourceLocalFile();
            }
            else {
                console.log("type error");
                return Promise.reject("type error");
            }
            return this._local_.init(path);
        });
    }
    /**
     * 注册资源
     * @param type
     * @param name
     * @param path
     */
    setResource(pack, name, data) {
        return __awaiter(this, void 0, void 0, function* () {
            let md5 = crypto_1.createHash('md5').update(data).digest('hex');
            let info = yield this.getResource(pack, name);
            if (info) {
                if (info.md5 == md5)
                    return true;
                info.data = data;
                info.md5 = md5;
            }
            return yield this._local_.saveResource(pack, name, data);
        });
    }
    getResource(pack, name) {
        return __awaiter(this, void 0, void 0, function* () {
            let id = pack + '_' + name;
            if (this._cache.hasOwnProperty(id)) {
                return this._cache.get(id);
            }
            try {
                let info = yield this._local_.getResource(pack, name);
                this._cache.set(id, info);
                return info;
            }
            catch (e) {
            }
            return undefined;
        });
    }
    /**
     * 对应 check 请求
     * @param file
     * @param md5
     */
    checkResource(pack, file, md5) {
        return __awaiter(this, void 0, void 0, function* () {
            let v = yield this.getResource(pack, file);
            if (v && v.md5 != md5) {
                return { code: 1 };
            }
            return { code: 0 };
        });
    }
    /**
     * 对应download请求
     * @param file
     */
    download(pack, file) {
        return __awaiter(this, void 0, void 0, function* () {
            let v = yield this.getResource(pack, file);
            if (v) {
                return { code: 0, text: v.data };
            }
            else {
                return { code: 1 };
            }
        });
    }
}
exports.ResourceService = ResourceService;
