"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// rsa - des
const node_rsa_1 = __importDefault(require("node-rsa"));
const crypto_1 = require("crypto");
// 默认的 formate 是 pkcs8 的
// 默认的 encryptionScheme 是 pkcs1_oaep
class RsaAesClient {
    constructor(size = 2048, encryptionScheme) {
        this.AZaz09 = '';
        this.option = {
            encryptionScheme: 'pkcs1_oaep',
            formatpem: 'pkcs8-public',
            algorithm: 'aes-128-cbc',
            size: 1024
        };
        for (let i = 'A'.charCodeAt(0); i < 'Z'.charCodeAt(0); i++) {
            this.AZaz09 += String.fromCharCode(i);
        }
        for (let i = 'a'.charCodeAt(0); i < 'z'.charCodeAt(0); i++) {
            this.AZaz09 += String.fromCharCode(i);
        }
        for (let i = '0'.charCodeAt(0); i < '9'.charCodeAt(0); i++) {
            this.AZaz09 += String.fromCharCode(i);
        }
        this.useKeys = { client: new node_rsa_1.default, server: new node_rsa_1.default };
        if (encryptionScheme)
            this.option.encryptionScheme = encryptionScheme;
        this.useKeys.client.setOptions({
            encryptionScheme: this.option.encryptionScheme
        });
        if (size)
            this.option.size = size;
        this._parseAlgorithm();
    }
    loadAesKey(key, algorithm) {
        this.useKeys.aes = { algorithm: algorithm || this.option.algorithm, key: key, autoPadding: true };
        this._parseAlgorithm();
    }
    loadAes(aes) {
        this.useKeys.aes = aes;
        this._parseAlgorithm();
    }
    _parseAlgorithm() {
        let algorithms = (this.useKeys.aes ? this.useKeys.aes.algorithm : this.option.algorithm).toUpperCase().split('-');
        this.algorithmOption = { mode: algorithms[2], len: parseInt(algorithms[1]) / 8, type: algorithms[0] };
    }
    randWord(len = 1) {
        let out = '';
        for (let i = 0; i < len; i++) {
            let num = Math.floor(Math.random() * this.AZaz09.length);
            out += this.AZaz09.charAt(num);
        }
        return out;
    }
    /**
     * 生成rsa 证书信息 返回公钥内容
     */
    generateRsa() {
        if (this.useKeys.client.isEmpty())
            this.useKeys.client.generateKeyPair(this.option.size);
        return this.useKeys.client.exportKey(this.option.formatpem);
    }
    /**
     * 装在rsa公钥信息 返回NodeRsa对象
     * @param key
     */
    loadPublicRsa(key, format, encryptionScheme) {
        if (this.useKeys.server.isEmpty()) {
            this.useKeys.server.importKey(key, format || this.option.formatpem);
            this.useKeys.server.setOptions({
                encryptionScheme: encryptionScheme || this.option.encryptionScheme
            });
        }
        return this.useKeys.server;
    }
    stepOne(pbkey) {
        // let pkey = inflateRawSync(pbkey).toString();
        let pkey = pbkey.toString();
        // 接收明文证书
        let rsa = this.loadPublicRsa(pkey);
        // 生成自己的证书
        let key = this.generateRsa();
        // 把自己的证书内容发送过去
        return rsa.encrypt(key);
    }
    stepTwo(data) {
        // 接受收对称加密的密钥
        let eInfo = this.useKeys.client.decrypt(data);
        this.useKeys.aes = JSON.parse(eInfo.toString());
        return true;
    }
    // 对称加密数据
    encode(data) {
        if (!this.useKeys.aes)
            throw Error('no aes generate');
        else {
            let iv = this.randWord(16);
            if (this.algorithmOption.mode == 'ECB')
                iv = null;
            let decipher = crypto_1.createCipheriv(this.useKeys.aes.algorithm, this.useKeys.aes.key, iv);
            decipher.setAutoPadding(this.useKeys.aes.autoPadding);
            let encryptedData = decipher.update(data, 'utf8', 'base64');
            encryptedData += decipher.final('base64');
            return { encryptedData: encryptedData, iv: iv };
        }
    }
    // 对称解密数据
    decode(encryptedData, iv) {
        if (!this.useKeys.aes)
            throw Error('no aes generate');
        else {
            if (!iv || this.algorithmOption.mode == 'ECB')
                iv = null;
            let decipher = crypto_1.createDecipheriv(this.useKeys.aes.algorithm, this.useKeys.aes.key, iv);
            decipher.setAutoPadding(this.useKeys.aes.autoPadding);
            let decryptData = decipher.update(encryptedData, 'base64', 'utf8');
            decryptData += decipher.final('utf8');
            return decryptData;
        }
    }
}
exports.RsaAesClient = RsaAesClient;
