"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// rsa - des
const node_rsa_1 = __importDefault(require("node-rsa"));
const crypto_1 = require("crypto");
// 默认的 formate 是 pkcs8 的
// 默认的 encryptionScheme 是 pkcs1_oaep
class RsaAesServer {
    /**
     * 初始化一个密钥流程
     * @param size 密钥长度
     * @param encryptionScheme 如果是提供给 web的 jsencrypt使用需要填 pkcs1
     * @param bSameServer 是否使用统一的服务端私钥，这会大大加速公钥的获取速度
     */
    constructor(size = 1024, encryptionScheme, bSameServer = false) {
        this.AZaz09 = '';
        this.option = {
            encryptionScheme: 'pkcs1_oaep',
            formatpem: 'pkcs8-public',
            algorithm: 'aes-128-cbc',
            size: 1024
        };
        for (let i = 'A'.charCodeAt(0); i < 'Z'.charCodeAt(0); i++) {
            this.AZaz09 += String.fromCharCode(i);
        }
        for (let i = 'a'.charCodeAt(0); i < 'z'.charCodeAt(0); i++) {
            this.AZaz09 += String.fromCharCode(i);
        }
        for (let i = '0'.charCodeAt(0); i < '9'.charCodeAt(0); i++) {
            this.AZaz09 += String.fromCharCode(i);
        }
        if (encryptionScheme)
            this.option.encryptionScheme = encryptionScheme;
        if (size)
            this.option.size = size;
        this.useKeys = { server: bSameServer ? RsaAesServer.getServerNode(size) : new node_rsa_1.default, client: new node_rsa_1.default };
        this.useKeys.server.setOptions({
            encryptionScheme: this.option.encryptionScheme,
        });
        this._parseAlgorithm();
    }
    static getServerNode(size) {
        if (!this.server[size]) {
            this.server[size] = new node_rsa_1.default; //.generateKeyPair(size);
        }
        return this.server[size];
    }
    loadAesKey(key, algorithm) {
        this.useKeys.aes = { algorithm: algorithm || this.option.algorithm, key: key, autoPadding: true };
        this._parseAlgorithm();
    }
    loadAes(aes) {
        this.useKeys.aes = aes;
        this._parseAlgorithm();
    }
    _parseAlgorithm() {
        let algorithms = (this.useKeys.aes ? this.useKeys.aes.algorithm : this.option.algorithm).toUpperCase().split('-');
        this.algorithmOption = { mode: algorithms[2], len: parseInt(algorithms[1]) / 8, type: algorithms[0] };
    }
    randWord(len = 1) {
        let out = '';
        for (let i = 0; i < len; i++) {
            let num = Math.floor(Math.random() * this.AZaz09.length);
            out += this.AZaz09.charAt(num);
        }
        return out;
    }
    generateRsa(format) {
        if (this.useKeys.server.isEmpty())
            this.useKeys.server.generateKeyPair(this.option.size);
        return this.useKeys.server.exportKey(format || this.option.formatpem);
    }
    loadPublicRsa(key, format, encryptionScheme) {
        if (this.useKeys.client.isEmpty()) {
            this.useKeys.client.importKey(key, format || this.option.formatpem);
            this.useKeys.client.setOptions({
                encryptionScheme: encryptionScheme || this.option.encryptionScheme
            });
        }
        return this.useKeys.client;
    }
    generateDes() {
        if (!this.useKeys.aes) {
            this.useKeys.aes = { algorithm: this.option.algorithm, key: this.randWord(this.algorithmOption.len), autoPadding: true };
        }
        return this.useKeys.aes;
    }
    getAesKey() {
        return this.useKeys.aes;
    }
    /**
     * 第一步，提供公钥给客户端
     * @param format
     */
    stepOne(format) {
        if (format && format.indexOf('public') < 0) {
            // 如果想偷私钥，那么回复 public过去
            return this.generateRsa('public');
        }
        // 直接明文发证书
        // return deflateRawSync(this.generateRsa());
        return this.generateRsa(format);
    }
    /**
     * 接收客户端使用公钥加密过的新的公钥,然后返回  加密的 aes 密钥信息
     * @param data
     * @param format
     * @param encryptionScheme
     */
    stepTwo(data, format, encryptionScheme) {
        // 先解密数据
        let pdata = this.useKeys.server.decrypt(data).toString();
        // 装在客户端发过来的证书内容
        let rsa = this.loadPublicRsa(pdata, format, encryptionScheme);
        // 用前端的公钥加密后发送
        let buff = JSON.stringify(this.generateDes());
        // let per_size = 256
        // let chunk = Math.ceil(buff.length / per_size);
        // let bfs: Buffer[] = []
        // for (let i = 0; i < chunk; i++) {
        //     bfs.push(rsa.encrypt(buff.slice(i * per_size, (i + 1) * per_size)));
        // }
        console.log(buff);
        return rsa.encrypt(buff);
    }
    // 数据测试接口
    testDecrypt(type, wds) {
        switch (type) {
            case 'server': return this.useKeys.server.decrypt(wds);
            case 'client': return this.useKeys.client.decryptPublic(wds);
            default: throw {};
        }
    }
    // 数据测试接口
    testEncrypt(type, wds) {
        switch (type) {
            case 'server': return this.useKeys.server.encryptPrivate(wds);
            case 'client': return this.useKeys.client.encrypt(wds);
            default: throw {};
        }
    }
    // 对称加密数据
    encode(data) {
        if (!this.useKeys.aes)
            throw Error('no aes generate');
        else {
            let iv = this.randWord(16);
            if (this.algorithmOption.mode == 'ECB')
                iv = null;
            let decipher = crypto_1.createCipheriv(this.useKeys.aes.algorithm, this.useKeys.aes.key, iv);
            decipher.setAutoPadding(this.useKeys.aes.autoPadding);
            let encryptedData = decipher.update(data, 'utf8', 'base64');
            encryptedData += decipher.final('base64');
            return { encryptedData: encryptedData, iv: iv };
        }
    }
    // 对称解密数据
    decode(encryptedData, iv) {
        if (!this.useKeys.aes)
            throw Error('no aes generate');
        else {
            if (!iv || this.algorithmOption.mode == 'ECB')
                iv = null;
            let decipher = crypto_1.createDecipheriv(this.useKeys.aes.algorithm, this.useKeys.aes.key, iv);
            decipher.setAutoPadding(this.useKeys.aes.autoPadding);
            let decryptData = decipher.update(encryptedData, 'base64', 'utf8');
            decryptData += decipher.final('utf8');
            return decryptData;
        }
    }
}
exports.RsaAesServer = RsaAesServer;
RsaAesServer.server = {};
