"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("./server");
const express_1 = __importDefault(require("express"));
const fs_1 = require("fs");
const path_1 = require("path");
const route_1 = require("./route");
const swagger_1 = require("./swagger");
var instance = new server_1.cWebRouteModule();
/**
 * 这里负责http的解析，然后传递给对于的处理方法
 * 目前比较简陋，后续需要增加一下处理
 */
exports.WebRouteModule = {
    name: 'WebRouteModule',
    instance: instance,
    /**在模块初始化开始前注册 */
    use(...handlers) {
        if (instance.inited) {
            // console.warn('has b')
            return false;
        }
        else {
            instance.usePools.push(...handlers);
            return true;
        }
    },
    /**帮助文档 教你怎么填配置信息 */
    SwaggerHelp: swagger_1.SwaggerHelp,
    /**在模块初始化开始前注册 */
    enableStatic: function (root) {
        if (instance.inited) {
            // console.warn('has b')
            return false;
        }
        else {
            instance.usePools.push(express_1.default.static(root));
            return true;
        }
    },
    /**注册代码模块的位置 */
    routePath(routePath) {
        instance._route_path = routePath;
    },
    docPath(mdfilePath) {
        if (fs_1.existsSync(mdfilePath)) {
            instance._docPath = mdfilePath;
        }
    },
    /**
     * 是否开启跨域
     */
    openCross(b_switch = true) {
        instance.openCross = b_switch;
    },
    /**
     * @description 开启Swagger模块
     * @date 2020-01-16
     * @param {{ basedir: string; jsPath: string; description?: string; title?: string; version?: string; host?: string; basePath?: string; produces?: string[]; schemes?: string[]; }} { basedir, jsPath, description = 'This is a sample server', title = 'Swagger', version = '1.0.0', host = 'localhost', basePath = '', produces = [
     *             "application/json"
     *         ], schemes = ['http'] }
     */
    openSwagger({ basedir, jsPath, description = 'http api server', title = 'dreamduke', version = '1.0.0', host, basePath = '', produces = [
        "application/json"
    ], schemes = ['http'] }) {
        let option = {
            swaggerDefinition: {
                info: {
                    description: description,
                    title: title,
                    version: version
                },
                host: host,
                basePath: basePath,
                produces: produces,
                schemes: schemes,
                securityDefinitions: {
                    JWT: {
                        type: 'apiKey',
                        in: 'header',
                        name: 'Authorization',
                        description: "",
                    }
                }
            },
            route: {
                url: '/swagger',
                docs: '/swagger.json',
            },
            basedir: basedir || process.cwd(),
            files: [path_1.join(jsPath || './web', '/**/*.js')] //Path to the API handle folder
        };
        instance.openSwagger(option);
    },
    /**
     * 注册响应函数
     * @param dirname 接口文件地址
     * @param event 接口名字
     * @param paramlist 提前检查的参数 不满足会提前返回失败
     * @param func 处理函数
     */
    registRoute(dirname, event, paramlist, func) {
        route_1.regist(dirname, event, paramlist, func);
    },
    /**
     * 初始化模块
     * @param port
     */
    init(port, limit) {
        return new Promise(function (resolve) {
            instance.init(port, limit, resolve);
        });
    }
};
