"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const bodyParser = require("body-parser");
const fs_1 = require("fs");
const path_1 = require("path");
const zlib_1 = require("zlib");
const debug_1 = require("debug");
let Debug = debug_1.debug("server-libs");
class cWebRouteModule {
    constructor() {
        this.usePools = [];
        this._route_path = '';
        this.app = express_1.default();
        this.openCross = false;
        this._docPath = '';
        this.inited = false;
        this.favicon_data = Buffer.from("");
        if (process.mainModule) {
            let _p = path_1.parse(process.mainModule.filename);
            this._route_path = path_1.join(_p.dir, 'src', 'web');
        }
        else {
            this._route_path = process.cwd();
        }
    }
    use(...handlers) {
        if (this.app)
            this.app.use(...handlers);
    }
    init(port, limit, finish) {
        if (this.inited)
            return finish && finish();
        // 建立一个web监控
        let server = this.app.listen(port);
        server.on("error", function (e) {
            Debug(e);
        });
        server.on("listening", () => {
            Debug("listen on:" + port);
            this.app.use((req, res, next) => {
                if (this.openCross) {
                    res.header("Access-Control-Allow-Origin", "*");
                    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
                    res.header("Access-Control-Allow-Methods", "GET,POST,OPTIONS");
                }
                next();
            });
            this.app.options("*", function (req, res) {
                res.end();
            });
            if (typeof limit == 'string') {
                limit = {
                    text: limit,
                    json: limit,
                    urlencoded: limit,
                    raw: limit
                };
            }
            else if (limit == undefined) {
                limit = {};
            }
            this.app.use(bodyParser.text({ limit: limit.text || '100kb' }));
            this.app.use(bodyParser.raw({ limit: limit.raw || '100kb' }));
            this.app.use(bodyParser.urlencoded({ extended: true, limit: limit.urlencoded || "10mb" }));
            this.app.use(bodyParser.json({ limit: limit.json || '100kb' }));
            // 注册一下别人提交的内容
            for (let i = 0; i < this.usePools.length; i++) {
                this.app.use(this.usePools[i]);
            }
            this.app.use(this.favicon.bind(this));
            this.app.use(this.router.bind(this));
            this.app.use(function (req, res, next) {
                if (!res.headersSent) {
                    // 表示有人处理过了
                    res.writeHead(404);
                    res.write(JSON.stringify({
                        code: -1,
                        errorMsg: "can not find [" + req.path + "]"
                    }));
                }
                res.end();
            });
            finish && finish();
        });
        this.load_favicon();
        this.inited = true;
    }
    openSwagger(options) {
        try {
            const expressSwagger = require('express-swagger-generator')(this.app);
            expressSwagger(options);
        }
        catch (e) {
            if (e && (e instanceof Error)) {
                throw new Error('need install node_module first: npm install ' + (e.message.match(/\'.*\'/) || []).join(' ').replace(/\'/g, ''));
            }
        }
    }
    router(req, res, next) {
        // 这里处理一下传递参数的问题统一放在 req.params 上面
        req.params = Object.assign(req.params, req.query || {}, req.body || {});
        res.setHeader("Content-Type", "text/html; charset=utf-8");
        for (let key in req.params) {
            if (req.params[key] == "undefined" || req.params[key] == "null") {
                delete req.params[key];
            }
        }
        // console.log(req.url);
        // 先清理?之前的
        let sepIdx = req.url.indexOf("?");
        let tureUrl = req.url;
        if (sepIdx > 0) {
            tureUrl = req.url.slice(0, sepIdx);
        }
        let pathList = tureUrl.split("/");
        // 把里面空的都抛弃掉
        for (let i = 0; i < pathList.length; i++)
            if (!pathList[i]) {
                pathList.splice(i, 1);
                i--;
            }
        // deepRoouter(this._route_path, pathList, req, res, next);
        let [apifile, apifun] = pathList;
        try {
            let modulePath = apifile ? path_1.join(this._route_path, apifile) : this._route_path;
            let func;
            try {
                // 可能出现一种情况，接口直接卸载 index.js 中
                func = require(modulePath);
            }
            catch (e) {
                // apifile, apifun
                if (apifile) {
                    apifun = apifile;
                    func = require(this._route_path);
                    pathList.unshift('');
                }
            }
            if (func && apifun && func[apifun]) {
                func[apifun](req, res, next);
            }
            else if (func && func.router) {
                func.router(pathList.slice(1, pathList.length), req, res, next);
            }
            else {
                next();
            }
        }
        catch (e) {
            next();
        }
    }
    load_favicon() {
        try {
            this.favicon_data = zlib_1.gzipSync(fs_1.readFileSync(path_1.join(this._route_path, "favicon.ico")));
        }
        catch (e) { }
    }
    favicon(req, res, next) {
        if (req.url == "/favicon.ico") {
            res.writeHead(200, {
                "Content-Type": "image/x-icon",
                "Content-Encoding": "gzip"
            });
            res.write(this.favicon_data);
            res.end();
        }
        else {
            next();
        }
    }
}
exports.cWebRouteModule = cWebRouteModule;
